/***************************************************************************
 *   Copyright (C) 2010-2012 by Stephane List                              *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
  Example of arp-scan

root@Ubuntu-desktop:~# arp-scan -l
Interface: eth0, datalink type: EN10MB (Ethernet)
Starting arp-scan 1.6 with 256 hosts (http://www.nta-monitor.com/tools/arp-scan/)
192.168.1.1     00:17:33:1e:2b:74       neuf cegetel
192.168.1.44    00:04:30:32:d7:0c       Netgem
192.168.1.62    00:20:00:62:79:66       LEXMARK INTERNATIONAL, INC.
192.168.1.71    00:40:8c:99:f5:72       AXIS COMMUNICATIONS AB
192.168.1.72    00:1b:67:00:9b:50       Ubiquisys Ltd

5 packets received by filter, 0 packets dropped by kernel
Ending arp-scan 1.6: 256 hosts scanned in 2.339 seconds (109.45 hosts/sec).  5 responded

*/

#include "dialog.h"
#include "ui_dialog.h"
#include <QStringList>
#include <QProcess>
#include <QFile>
#include <QList>
#include <QByteArray>
#include <QNetworkInterface>

#include <QDebug>

#include "version.h"
#include "test_port.h"
#include "button_http.h"
#include "button_rtsp.h"
#include "button_ssh.h"

const quint16 PORT_HTTP = 80;
const quint16 PORT_SSH = 22;
const quint16 PORT_RTSP = 554;

Dialog::Dialog(QWidget *parent) :
        QDialog(parent),
        ScanProcess(0),
        ui(new Ui::Dialog)

{
    ui->setupUi(this);
    setWindowTitle(windowTitle() + " (V" + VERSION + ")");
#if 1
    if (!check_root())
    {
        ui->ScanButton->setText(tr("Only user Root can scan (Launch gksudo /path/to/qt-arp-scan in your desktop icon)"));
        ui->ScanButton->setEnabled(false);
        ui->comboBox->deleteLater();
        ui->ScanTable->deleteLater();
        ui->ScanResult->deleteLater();
        return;
    }
#endif
    if (!check_arp_scan())
    {
        ui->ScanButton->setText(tr("arp-scan is not installed"));
        ui->ScanButton->setEnabled(false);

        ui->ScanTable->deleteLater();
        ui->ScanResult->deleteLater();
        return;
    }

    QList<QNetworkInterface> lni = QNetworkInterface::allInterfaces ();
    QNetworkInterface ni;
    ui->comboBox->clear();

    Q_FOREACH(ni, lni) {
        quint8 flag = ni.flags();
        if (flag | QNetworkInterface::IsUp) {
            if ((flag & QNetworkInterface::IsLoopBack) == 0) {
                QNetworkAddressEntry nae;
                bool ipv4 = false;
                Q_FOREACH(nae, ni.addressEntries()) {
                    if (nae.ip().protocol() == QAbstractSocket::IPv4Protocol) {
                        QString color;
                        if (nae.prefixLength()<24) {
                           color = "#ff0000"; // red
                        }
                        else {
                            color = "#000000"; // black
                        }
                        ui->ScanResult->appendHtml("<p>" + ni.name() + " " + nae.ip().toString() + "<span style=\" color:" + color + ";\">"  + QString("/%1").arg(nae.prefixLength()) + "</span></p>");
                        ipv4 = true;
                    }
                }
                if (ipv4) {
                    //qDebug() << nae.ip();
                    ui->comboBox->addItem(ni.name());
                }
            }
        }
    }
    // Use eth0 as default interface
    int ethzero = ui->comboBox->findText("eth0");
    if (ethzero>=0) {
        ui->comboBox->setCurrentIndex(ethzero);
    }

}

bool Dialog::check_root()
{
    QStringList environment = QProcess::systemEnvironment();

    if (environment.contains("USER=root"))
    {
        return true;
    }
    return false;
}

bool Dialog::check_arp_scan()
{
    QFile file("/usr/bin/arp-scan");
    if (file.exists())
        return true;
    else
        return false;
}

Dialog::~Dialog()
{
    delete ui;
}

void Dialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void Dialog::on_ScanButton_clicked()
{
    ui->ScanButton->setText(tr("Scanning..."));
    ui->ScanButton->setEnabled(false);

    while (ui->ScanTable->rowCount()>0)
        ui->ScanTable->removeRow(0);

    if (ScanProcess == NULL)
    {
        ScanProcess = new QProcess();
        connect(ScanProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(On_Scan_Output()));
        connect(ScanProcess, SIGNAL(readyReadStandardError()), this, SLOT(On_Scan_Error()));
        connect(ScanProcess, SIGNAL(finished(int)), this, SLOT(On_Scan_Finished(int)));
    }

    QString ni = ui->comboBox->currentText();
    ui->ScanResult->appendPlainText("/usr/bin/arp-scan -l -I " + ni);

    ScanProcess->start("/usr/bin/arp-scan", QStringList() << "-l" << "-I" << ni);
}


void Dialog::On_Scan_Output()
{
    QList<QByteArray> list=ScanProcess->readAllStandardOutput().split('\n');
    QByteArray line;

    // for each output line of type :
    // 192.168.1.71    11:12:13:14:15:16       Vendor XYZ
    Q_FOREACH(line, list)
    {
        ui->ScanResult->appendPlainText(line);
        QList<QByteArray> list2=line.split('\t');
        if (list2.size()==3)
        {
            // new table line entry
            int i=ui->ScanTable->rowCount();
            ui->ScanTable->insertRow(i);
            for (int j=0;j<3;j++)
            {
                ui->ScanTable->setItem(i, j, new QTableWidgetItem(QString(list2[j])));
            }

            test_port *t1, *t2, *t3;
            t1 = new test_port(this);
            t2 = new test_port(this);
            t3 = new test_port(this);
            connect(t1, SIGNAL(result(const QString &, quint16, bool)), this, SLOT(On_Test_Result(const QString &, quint16, bool)));
            connect(t2, SIGNAL(result(const QString &, quint16, bool)), this, SLOT(On_Test_Result(const QString &, quint16, bool)));
            connect(t3, SIGNAL(result(const QString &, quint16, bool)), this, SLOT(On_Test_Result(const QString &, quint16, bool)));
            t1->run_test(QString(list2[0]),PORT_HTTP);
            t2->run_test(QString(list2[0]),PORT_SSH);
            t3->run_test(QString(list2[0]),PORT_RTSP);

        }
    }
    for (int j=0; j<3;j++)
    {
        ui->ScanTable->resizeColumnToContents(j);
    }
}

void Dialog::On_Scan_Error()
{
    ui->ScanResult->appendPlainText(ScanProcess->readAllStandardError());
}

void Dialog::On_Scan_Finished(int code)
{
    if (code>=0)
        ui->ScanResult->appendPlainText(tr("Finished."));
    else
        ui->ScanResult->appendPlainText(tr("Finished with errors."));
    ui->ScanButton->setText(tr("Scan"));
    ui->ScanButton->setEnabled(true);
}

void Dialog::On_Test_Result(const QString & ip, quint16 port, bool res)
{
    // search line in table
    for (int i=0;i<ui->ScanTable->rowCount();i++)
    {
        if (res && (ip == ui->ScanTable->item(i, 0)->text()))
        {
            switch (port) {
            case PORT_HTTP:
                {
                    button_http *b = new button_http(this);
                    b->set_ip(ip);
                    ui->ScanTable->setCellWidget(i, 3, b);
                }
                break;
            case PORT_SSH:
                {
                    button_ssh *b = new button_ssh(this);
                    b->set_ip(ip);
                    ui->ScanTable->setCellWidget(i, 4, b);
                }
                break;
            case PORT_RTSP:
                {
                    button_rtsp *b = new button_rtsp(this);
                    b->set_ip(ip);
                    ui->ScanTable->setCellWidget(i, 5, b);
                }
                break;
            default:
                qWarning("This port should not be scanned !");
            }
            break;
        }
    }
}
